@@ -1,12 +1,18 @@
 # This file is part of ranger, the console file manager.
 # License: GNU GPL version 3, see the file "AUTHORS" for details.
 
+from __future__ import (absolute_import, division, print_function)
+
 import string
 import re
 import os
+
+from ranger.core.shared import FileManagerAware
+
 ALLOWED_KEYS = string.ascii_letters + string.digits + "`'"
 
-class Bookmarks(object):
+
+class Bookmarks(FileManagerAware):
     """Bookmarks is a container which associates keys with bookmarks.
 
     A key is a string with: len(key) == 1 and key in ALLOWED_KEYS.
@@ -22,7 +28,8 @@ class Bookmarks(object):
     autosave = True
     load_pattern = re.compile(r"^[\d\w']:.")
 
-    def __init__(self, bookmarkfile, bookmarktype=str, autosave=False):
+    def __init__(self, bookmarkfile, bookmarktype=str, autosave=False,
+                 nonpersistent_bookmarks=()):
         """Initializes Bookmarks.
 
         <bookmarkfile> specifies the path to the file where
@@ -30,26 +37,19 @@ class Bookmarks(object):
         """
         self.autosave = autosave
         self.dct = {}
+        self.original_dict = {}
         self.path = bookmarkfile
         self.bookmarktype = bookmarktype
+        self.nonpersistent_bookmarks = set(nonpersistent_bookmarks)
 
     def load(self):
         """Load the bookmarks from path/bookmarks"""
-        try:
-            new_dict = self._load_dict()
-        except OSError:
+        new_dict = self._load_dict()
+        if new_dict is None:
             return
 
         self._set_dict(new_dict, original=new_dict)
 
-    def delete(self, key):
-        """Delete the bookmark with the given key"""
-        if key == '`':
-            key = "'"
-        if key in self.dct:
-            del self.dct[key]
-            if self.autosave: self.save()
-
     def enter(self, key):
         """Enter the bookmark with the given key.
 
@@ -68,7 +68,17 @@ class Bookmarks(object):
     def remember(self, value):
         """Bookmarks <value> to the key '"""
         self["'"] = value
-        if self.autosave: self.save()
+        if self.autosave:
+            self.save()
+
+    def __delitem__(self, key):
+        """Delete the bookmark with the given key"""
+        if key == '`':
+            key = "'"
+        if key in self.dct:
+            del self.dct[key]
+            if self.autosave:
+                self.save()
 
     def __iter__(self):
         return iter(self.dct.items())
@@ -78,7 +88,11 @@ class Bookmarks(object):
         if key == '`':
             key = "'"
         if key in self.dct:
-            return self.dct[key]
+            value = self.dct[key]
+            if self._validate(value):
+                return value
+            else:
+                raise KeyError("Cannot open bookmark: `%s'!" % key)
         else:
             raise KeyError("Nonexistant Bookmark: `%s'!" % key)
 
@@ -92,25 +106,38 @@ class Bookmarks(object):
             key = "'"
         if key in ALLOWED_KEYS:
             self.dct[key] = value
-            if self.autosave: self.save()
+            if self.autosave:
+                self.save()
 
     def __contains__(self, key):
         """Test whether a bookmark-key is defined"""
         return key in self.dct
 
+    def update_path(self, path_old, file_new):
+        """Update bookmarks containing path"""
+        self.update_if_outdated()
+        changed = False
+        for key, bfile in self:
+            if bfile.path == path_old:
+                self.dct[key] = file_new
+                changed = True
+            elif bfile.path.startswith(path_old + os.path.sep):
+                self.dct[key] = self.bookmarktype(file_new.path + bfile.path[len(path_old):])
+                changed = True
+        if changed:
+            self.save()
+
     def update(self):
         """Update the bookmarks from the bookmark file.
 
         Useful if two instances are running which define different bookmarks.
         """
-
-        try:
-            real_dict = self._load_dict()
-            real_dict_copy = real_dict.copy()
-        except OSError:
+        real_dict = self._load_dict()
+        if real_dict is None:
             return
+        real_dict_copy = real_dict.copy()
 
-        for key in set(self.dct.keys()) | set(real_dict.keys()):
+        for key in set(self.dct) | set(real_dict):
             # set some variables
             if key in self.dct:
                 current = self.dct[key]
@@ -145,50 +172,71 @@ class Bookmarks(object):
         self.update()
         if self.path is None:
             return
-        if os.access(self.path, os.W_OK):
-            f = open(self.path+".new", 'w')
-            for key, value in self.dct.items():
-                if type(key) == str\
-                        and key in ALLOWED_KEYS:
-                    try:
-                        f.write("{0}:{1}\n".format(str(key), str(value)))
-                    except:
-                        pass
 
-            f.close()
+        path_new = self.path + '.new'
+        try:
+            fobj = open(path_new, 'w')
+        except OSError as ex:
+            self.fm.notify('Bookmarks error: {0}'.format(str(ex)), bad=True)
+            return
+        for key, value in self.dct.items():
+            if isinstance(key, str) and key in ALLOWED_KEYS \
+                    and key not in self.nonpersistent_bookmarks:
+                fobj.write("{0}:{1}\n".format(str(key), str(value)))
+        fobj.close()
+
+        try:
             old_perms = os.stat(self.path)
-            try:
-                os.chown(self.path+".new", old_perms.st_uid, old_perms.st_gid)
-                os.chmod(self.path+".new", old_perms.st_mode)
-            except OSError:
-                pass
-            os.rename(self.path+".new", self.path)
+            os.chown(path_new, old_perms.st_uid, old_perms.st_gid)
+            os.chmod(path_new, old_perms.st_mode)
+
+            if os.path.islink(self.path):
+                target_path = os.path.realpath(self.path)
+                os.rename(path_new, target_path)
+            else:
+                os.rename(path_new, self.path)
+
+        except OSError as ex:
+            self.fm.notify('Bookmarks error: {0}'.format(str(ex)), bad=True)
+            return
+
         self._update_mtime()
 
-    def _load_dict(self):
-        dct = {}
+    def enable_saving_backtick_bookmark(self, boolean):
+        """
+        Adds or removes the ' from the list of nonpersitent bookmarks
+        """
+        if boolean:
+            if "'" in self.nonpersistent_bookmarks:
+                self.nonpersistent_bookmarks.remove("'")  # enable
+        else:
+            self.nonpersistent_bookmarks.add("'")  # disable
 
+    def _load_dict(self):
         if self.path is None:
-            return dct
+            return {}
 
         if not os.path.exists(self.path):
             try:
-                f = open(self.path, 'w')
-            except:
-                raise OSError('Cannot read the given path')
-            f.close()
+                with open(self.path, 'w') as fobj:
+                    pass
+            except OSError as ex:
+                self.fm.notify('Bookmarks error: {0}'.format(str(ex)), bad=True)
+                return None
 
-        if os.access(self.path, os.R_OK):
-            f = open(self.path, 'r')
-            for line in f:
-                if self.load_pattern.match(line):
-                    key, value = line[0], line[2:-1]
-                    if key in ALLOWED_KEYS:
-                        dct[key] = self.bookmarktype(value)
-            f.close()
-            return dct
-        else:
-            raise OSError('Cannot read the given path')
+        try:
+            fobj = open(self.path, 'r')
+        except OSError as ex:
+            self.fm.notify('Bookmarks error: {0}'.format(str(ex)), bad=True)
+            return None
+        dct = {}
+        for line in fobj:
+            if self.load_pattern.match(line):
+                key, value = line[0], line[2:-1]
+                if key in ALLOWED_KEYS:
+                    dct[key] = self.bookmarktype(value)
+        fobj.close()
+        return dct
 
     def _set_dict(self, dct, original):
         if original is None:
@@ -209,3 +257,6 @@ class Bookmarks(object):
 
     def _update_mtime(self):
         self.last_mtime = self._get_mtime()
+
+    def _validate(self, value):  # pylint: disable=no-self-use
+        return os.path.isdir(str(value))
