@@ -175,4 +175,5 @@ def char_slice(ansi_text, start, length):
 
 if __name__ == '__main__':
     import doctest
-    doctest.testmod()
+    import sys
+    sys.exit(doctest.testmod()[0])
